@@ -5,6 +5,7 @@ from moddotplot.parse_fasta import (
     getInputHeaders,
     isValidFasta,
     extractFiles,
+    extractRegion,
 )
 
 from moddotplot.estimate_identity import (
@@ -12,6 +13,7 @@ from moddotplot.estimate_identity import (
     selfContainmentMatrix,
     pairwiseContainmentMatrix,
     convertMatrixToBed,
+    convertMatrixToCool,
     createSelfMatrix,
     createPairwiseMatrix,
     partitionOverlaps,
@@ -26,6 +28,7 @@ import json
 import numpy as np
 import pickle
 import os
+import cooler
 
 
 def get_parser():
@@ -35,7 +38,7 @@ def get_parser():
     """
     parser = argparse.ArgumentParser(
         formatter_class=argparse.ArgumentDefaultsHelpFormatter,
-        description="ModDotPlot: Visualization of Complex Repeat Structures",
+        description="ModDotPlot: Visualization of Tandem Repeats",
     )
     subparsers = parser.add_subparsers(
         dest="command", help="Choose mode: interactive or static"
@@ -176,8 +179,8 @@ def get_parser():
     )
 
     static_input_group.add_argument(
-        "-b",
-        "--bed",
+        "-l",
+        "--load",
         default=argparse.SUPPRESS,
         help="Path to input paired-end bed file(s). Exclusively used in static mode.",
         nargs="+",
@@ -197,6 +200,8 @@ def get_parser():
         required=False
     )
 
+    static_parser.add_argument("-b", "--bed", default=None, help="Bed file annotation.")
+
     static_parser.add_argument(
         "-k", "--kmer", default=21, type=int, help="k-mer length."
     )
@@ -261,7 +266,18 @@ def get_parser():
     )
 
     static_parser.add_argument(
-        "--no-bed", action="store_true", help="Skip output of bed file."
+        "--compare-order",
+        choices=["sequential", "size"],
+        default="sequential",
+        help="Order in which sequences appear in the comparative plot. Default is 'sequential': First file on x-axis, second file on y-axis. Another option is 'size': The larger sequence on the x-axis and the smaller on y-axis.",
+    )
+
+    static_parser.add_argument(
+        "--cooler", action="store_true", help="Output matrix to cooler file."
+    )
+
+    static_parser.add_argument(
+        "--no-bedpe", action="store_true", help="Skip output of paired-end bed file."
     )
 
     static_parser.add_argument(
@@ -273,10 +289,10 @@ def get_parser():
     )
 
     static_parser.add_argument(
-        "--width", default=18, type=float, help="Plot width (also height for _FULL)."
+        "--width", default=9, type=float, help="Plot width (also height for _FULL)."
     )
 
-    static_parser.add_argument("--dpi", default=600, type=int, help="Plot dpi.")
+    static_parser.add_argument("--dpi", default=300, type=int, help="Plot dpi.")
 
     # TODO: Create list of accepted colors.
     static_parser.add_argument(
@@ -423,6 +439,7 @@ def main():
                 config = json.load(f)
                 # TODO: Remove args that are interactive only
                 args.fasta = config.get("fasta")
+                args.load = config.get("load")
                 args.bed = config.get("bed")
 
                 # Distance matrix commands
@@ -436,7 +453,7 @@ def main():
                 args.compare = config.get("compare", args.compare)
                 args.compare_only = config.get("compare_only", args.compare_only)
 
-                args.no_bed = config.get("no_bed", args.no_bed)
+                args.no_bedpe = config.get("no_bedpe", args.no_bedpe)
                 args.no_plot = config.get("no_plot", args.no_plot)
                 args.no_hist = config.get("no_hist", args.no_hist)
                 args.width = config.get("width", args.width)
@@ -450,18 +467,13 @@ def main():
                 args.axes_ticks = config.get("axes_ticks", args.axes_ticks)
                 args.breakpoints = config.get("breakpoints", args.breakpoints)
                 args.bin_freq = config.get("bin_freq", args.bin_freq)
-
-        #
-        """if args.grid or args.grid_only:
-            if not (args.compare or args.compare_only) and not args.bed:
-                print(
-                    f"Option --grid was selected, but no comparative plots will be produced. Please rerun ModDotPlot with the `--compare` or `--compare-only` option.\n"
-                )
-                sys.exit(10)"""
+                args.axes_limits = config.get("axes_limits", args.axes_limits)
+                args.axes_ticks = config.get("axes_ticks", args.axes_ticks)
+                args.vector = config.get("vector", args.vector)
+                args.deraster = config.get("deraster", args.deraster)
 
         # -----------INPUT COMMAND VALIDATION-----------
         # TODO: More tests!
-        # Validation for breakpoints command
         if args.breakpoints:
             # Check that start value for breakpoints = identity threshold value
             if float(args.breakpoints[0]) != float(args.identity):
@@ -476,15 +488,15 @@ def main():
                 sys.exit(2)
 
         # -----------BEDFILE INPUT FOR STATIC MODE-----------
-        if hasattr(args, "bed") and args.bed:
+        if hasattr(args, "load") and args.load:
             if args.grid or args.grid_only:
                 single_vals = []
                 double_vals = []
                 single_val_name = []
                 double_val_name = []
                 xlim_val_grid = 0
-            for bed in args.bed:
-                # If args.bed is provided as input, run static mode directly from the bed file. Skip counting input k-mers.
+            for bed in args.load:
+                # If args.load is provided as input, run static mode directly from the paired-end bed file. Skip counting input k-mers.
                 df = read_df_from_file(bed)
 
                 unique_query_names = df["#query_name"].unique()
@@ -522,6 +534,7 @@ def main():
                             axes_tick_number=args.axes_number,
                             vector_format=args.vector,
                             deraster=args.deraster,
+                            annotation=args.bed,
                         )
                     if args.grid or args.grid_only:
                         single_vals.append(df)
@@ -559,6 +572,7 @@ def main():
                             axes_tick_number=args.axes_number,
                             vector_format=args.vector,
                             deraster=args.deraster,
+                            annotation=args.bed,
                         )
                     if args.grid or args.grid_only:
                         double_vals.append(df)
@@ -906,17 +920,27 @@ def main():
         if args.grid or args.grid_only:
             grid_val_singles = []
             grid_val_single_names = []
-        sequences = list(zip(seq_list, k_list))
+        new_sequences = list(zip(seq_list, k_list))
+        if args.compare_order == "size":
+            sequences = sorted(new_sequences, key=lambda seq: len(seq[1]), reverse=True)
+        else:
+            sequences = new_sequences
         if len(sequences) > 6 and (args.grid or args.grid_only):
             print("Too many sequences to create a grid. Skipping. \n")
 
         # Create output directory, if doesn't exist:
         if (args.output_dir) and not os.path.exists(args.output_dir):
-            os.makedirs(args.output_dir)
+            os.makedirs(args.output_dir, exist_ok=True)
         # -----------COMPUTE SELF-IDENTITY PLOTS-----------
         if not args.compare_only:
             for i in range(len(sequences)):
                 seq_length = len(sequences[i][1])
+                seq_name = sequences[i][0]
+                seq_range = extractRegion(seq_name)
+                if not seq_range:
+                    seq_start_pos = 1
+                else:
+                    seq_start_pos = int(seq_range[1])
                 win = args.window
                 res = args.resolution
                 if args.window:
@@ -941,10 +965,10 @@ def main():
                     seq_sparsity = 2 ** (int(math.log2(seq_sparsity - 1)) + 1)
                 expectation = round(win / seq_sparsity)
 
-                print(f"Computing self identity matrix for {sequences[i][0]}... \n")
+                print(f"Computing self identity matrix for {seq_name}... \n")
                 # TODO: Logging here
                 # print(f"\tSparsity value s: {seq_sparsity}\n")
-                print(f"\tSequence length n: {len(k_list[i]) + args.kmer - 1}\n")
+                print(f"\tSequence length n: {seq_length + args.kmer - 1}\n")
                 print(f"\tWindow size w: {win}\n")
                 print(f"\tModimizer sketch size: {expectation}\n")
                 print(f"\tPlot Resolution r: {res}\n")
@@ -960,31 +984,71 @@ def main():
                     expectation,
                 )
                 bed = convertMatrixToBed(
-                    self_mat, win, args.identity, seq_list[i], seq_list[i], True
+                    self_mat,
+                    win,
+                    args.identity,
+                    seq_name,
+                    seq_name,
+                    True,
+                    seq_start_pos,
+                    seq_start_pos,
                 )
                 if args.grid or args.grid_only:
                     grid_val_singles.append(bed)
-                    grid_val_single_names.append(sequences[i][0])
+                    grid_val_single_names.append(seq_name)
 
-                if not args.no_bed:
-                    # Log saving bed file
-                    if not args.output_dir:
-                        bedfile_output = sequences[i][0] + ".bed"
-                    else:
-                        bedfile_output = os.path.join(
-                            args.output_dir, sequences[i][0] + ".bed"
+                if args.cooler:
+                    try:
+                        cooler_path = "."
+                        if not args.output_dir:
+                            cooler_path = os.path.join(cooler_path, seq_name)
+                        else:
+                            cooler_path = os.path.join(args.output_dir, seq_name)
+                        os.makedirs(cooler_path, exist_ok=True)
+                        cooler_output = os.path.join(cooler_path, seq_name + ".cooler")
+                        convertMatrixToCool(
+                            matrix=self_mat,
+                            window_size=win,
+                            id_threshold=args.identity,
+                            x_name=seq_name,
+                            y_name=seq_name,
+                            self_identity=True,
+                            x_offset=seq_start_pos,
+                            y_offset=seq_start_pos,
+                            chromsizes=seq_length,
+                            output_cool=cooler_output,
                         )
+                        print(
+                            f"Saved self-identity matrix as a cooler file to {cooler_output}\n"
+                        )
+                    except Exception as e:
+                        print(f"Error creating cooler file: {e}")
+
+                if not args.no_bedpe:
+                    # Log saving bed file
+                    bedpe_path = "."
+                    if not args.output_dir:
+                        bedpe_path = os.path.join(bedpe_path, seq_name)
+                        os.makedirs(bedpe_path, exist_ok=True)
+                        bedfile_output = os.path.join(seq_name, seq_name + ".bedpe")
+                    else:
+                        bedpe_path = os.path.join(args.output_dir, seq_name)
+                        os.makedirs(bedpe_path, exist_ok=True)
+                        bedfile_output = os.path.join(bedpe_path, seq_name + ".bedpe")
+
                     with open(bedfile_output, "w") as bedfile:
                         for row in bed:
                             bedfile.write("\t".join(map(str, row)) + "\n")
-                    print(f"Saved bed file to {bedfile_output}\n")
+                    print(
+                        f"Saved self-identity matrix as a paired-end bed file to {bedfile_output}\n"
+                    )
 
                 if (not args.no_plot) and (not args.grid_only):
                     create_plots(
                         sdf=[bed],
-                        directory=args.output_dir if args.output_dir else ".",
-                        name_x=sequences[i][0],
-                        name_y=sequences[i][0],
+                        directory=bedpe_path,
+                        name_x=seq_name,
+                        name_y=seq_name,
                         palette=args.palette,
                         palette_orientation=args.palette_orientation,
                         no_hist=args.no_hist,
@@ -1000,6 +1064,7 @@ def main():
                         axes_tick_number=args.axes_number,
                         vector_format=args.vector,
                         deraster=args.deraster,
+                        annotation=args.bed,
                     )
 
         # -----------COMPUTE COMPARATIVE PLOTS-----------
@@ -1016,21 +1081,23 @@ def main():
 
             for i in range(len(sequences)):
                 for j in range(i + 1, len(sequences)):
+                    # Larger = x, smaller = y. This is pre-sorted earlier.
                     larger_seq = sequences[i][1]
                     smaller_seq = sequences[j][1]
                     larger_length = len(larger_seq)
                     smaller_length = len(smaller_seq)
                     larger_seq_name = sequences[i][0]
                     smaller_seq_name = sequences[j][0]
-
-                    if larger_length < smaller_length:
-                        smaller_seq = sequences[i][1]
-                        larger_seq = sequences[j][1]
-                        larger_length = len(larger_seq)
-                        smaller_length = len(smaller_seq)
-                        larger_seq_name = sequences[j][0]
-                        smaller_seq_name = sequences[i][0]
-
+                    larger_seq_range = extractRegion(larger_seq_name)
+                    if not larger_seq_range:
+                        larger_seq_start_pos = 1
+                    else:
+                        larger_seq_start_pos = int(larger_seq_range[1])
+                    smaller_seq_range = extractRegion(smaller_seq_name)
+                    if not larger_seq_range:
+                        smaller_seq_start_pos = 1
+                    else:
+                        smaller_seq_start_pos = int(smaller_seq_range[1])
                     win = args.window
                     res = args.resolution
                     if args.window:
@@ -1062,10 +1129,10 @@ def main():
                     print(f"\tPlot Resolution r: {res}\n")
 
                     pair_mat = createPairwiseMatrix(
-                        larger_length,
                         smaller_length,
-                        larger_seq,
+                        larger_length,
                         smaller_seq,
+                        larger_seq,
                         win,
                         seq_sparsity,
                         args.delta,
@@ -1080,13 +1147,51 @@ def main():
                             f"The pairwise identity matrix for {sequences[i][0]} and {sequences[j][0]} is empty. Skipping.\n"
                         )
                     else:
+                        if args.cooler:
+                            try:
+                                cooler_path = "."
+                                if not args.output_dir:
+                                    cooler_path = os.path.join(
+                                        cooler_path,
+                                        f"{larger_seq_name}_{smaller_seq_name}",
+                                    )
+                                else:
+                                    cooler_path = os.path.join(
+                                        args.output_dir,
+                                        f"{larger_seq_name}_{smaller_seq_name}",
+                                    )
+                                os.makedirs(cooler_path, exist_ok=True)
+                                cooler_output = os.path.join(
+                                    cooler_path,
+                                    f"{larger_seq_name}_{smaller_seq_name}.cooler",
+                                )
+                                convertMatrixToCool(
+                                    matrix=pair_mat,
+                                    window_size=win,
+                                    id_threshold=args.identity,
+                                    x_name=larger_seq_name,
+                                    y_name=smaller_seq_name,
+                                    self_identity=False,
+                                    x_offset=larger_seq_start_pos,
+                                    y_offset=smaller_seq_start_pos,
+                                    chromsizes=larger_length,
+                                    output_cool=cooler_output,
+                                )
+                                print(
+                                    f"Saved comparative matrix as a cooler file to {cooler_output}\n"
+                                )
+                            except Exception as e:
+                                print(f"Error creating pairwise cooler file: {e}")
                         bed = convertMatrixToBed(
                             pair_mat,
                             win,
                             args.identity,
-                            seq_list[i],
-                            seq_list[j],
+                            # check if this is correct
+                            larger_seq_name,
+                            smaller_seq_name,
                             False,
+                            larger_seq_start_pos,
+                            smaller_seq_start_pos,
                         )
                         if args.grid or args.grid_only:
                             grid_val_doubles.append(bed)
@@ -1094,34 +1199,47 @@ def main():
                                 [larger_seq_name, smaller_seq_name]
                             )
                             xlim_val_grid = max(larger_length, xlim_val_grid)
-                        if not args.no_bed:
+                        if not args.no_bedpe:
                             # Log saving bed file
+                            bedpe_path = "."
                             if not args.output_dir:
+                                bedfile_prefix = (
+                                    larger_seq_name + "_" + smaller_seq_name
+                                )
+                                bedpe_path = os.path.join(bedpe_path, bedfile_prefix)
+                                os.makedirs(bedpe_path, exist_ok=True)
                                 bedfile_output = (
-                                    smaller_seq_name
-                                    + "_"
-                                    + larger_seq_name
-                                    + "_COMPARE.bed"
+                                    bedpe_path,
+                                    bedfile_prefix + "_COMPARE.bedpe",
                                 )
                             else:
+                                bedfile_prefix = (
+                                    larger_seq_name + "_" + smaller_seq_name
+                                )
+                                bedpe_path = os.path.join(
+                                    args.output_dir, bedfile_prefix
+                                )
+                                os.makedirs(bedpe_path, exist_ok=True)
                                 bedfile_output = os.path.join(
-                                    args.output_dir,
-                                    smaller_seq_name
+                                    bedpe_path,
+                                    larger_seq_name
                                     + "_"
-                                    + larger_seq_name
-                                    + "_COMPARE.bed",
+                                    + smaller_seq_name
+                                    + "_COMPARE.bedpe",
                                 )
                             with open(bedfile_output, "w") as bedfile:
                                 for row in bed:
                                     bedfile.write("\t".join(map(str, row)) + "\n")
-                            print(f"Saved bed file to {bedfile_output}\n")
+                            print(
+                                f"Saved comparative matrix as a paired-end bed file to {bedfile_output}\n"
+                            )
 
                         if (not args.no_plot) and (not args.grid_only):
                             create_plots(
                                 sdf=[bed],
-                                directory=args.output_dir if args.output_dir else ".",
-                                name_x=smaller_seq_name,
-                                name_y=larger_seq_name,
+                                directory=bedpe_path,
+                                name_x=larger_seq_name,
+                                name_y=smaller_seq_name,
                                 palette=args.palette,
                                 palette_orientation=args.palette_orientation,
                                 no_hist=args.no_hist,
@@ -1137,6 +1255,7 @@ def main():
                                 axes_tick_number=args.axes_number,
                                 vector_format=args.vector,
                                 deraster=args.deraster,
+                                annotation=args.bed,
                             )
 
             if args.grid or args.grid_only:
