@@ -1,4 +1,3 @@
-from ast import excepthandler
 from plotnine import (
     ggsave,
     ggplot,
@@ -28,9 +27,11 @@ from plotnine import (
     theme_minimal,
     geom_raster,
 )
+import svgutils.transform as sg
 import cairosvg
 import pandas as pd
 import numpy as np
+import glob
 from PIL import Image
 import patchworklib as pw
 import math
@@ -39,7 +40,9 @@ import xml.etree.ElementTree as ET
 import sys
 import re
 from moddotplot.parse_fasta import printProgressBar
-
+from lxml import etree
+from pygenometracks.utilities import get_region
+import matplotlib.pyplot as plt
 from moddotplot.const import (
     DIVERGING_PALETTES,
     QUALITATIVE_PALETTES,
@@ -47,6 +50,18 @@ from moddotplot.const import (
 )
 from typing import List
 from palettable.colorbrewer import qualitative, sequential, diverging
+import logging
+
+# Set log level BEFORE importing pygenometracks
+for name in logging.root.manager.loggerDict:
+    if name.startswith("pygenometracks"):
+        logging.getLogger(name).setLevel(logging.CRITICAL)
+        logging.getLogger(name).propagate = False  # Don't pass to root logger
+
+# Also make sure the root logger isn’t outputting debug messages
+logging.basicConfig(level=logging.CRITICAL)
+
+from pygenometracks.tracksClass import PlotTracks
 
 
 def is_plot_empty(p):
@@ -75,6 +90,269 @@ def check_pascal(single_val, double_val):
         sys.exit(8)
 
 
+def generate_ini_file(
+    bedfile, ininame, chrom, color_value="bed_rgb", x_axis=True, display="collapsed"
+):
+    try:
+        thing = chrom.split(":")[0]
+        sections = [
+            "[spacer]",
+            "# height of space in cm (optional)",
+            "height = 0.5",
+            "",
+            f"[{thing}]",
+            f"file = {bedfile}",
+            f"Title=",
+            "height = 1",
+            f"display = {display}",
+            f"color = {color_value}",
+            "labels = false",
+            "fontsize = 10",
+            "file_type = bed",
+        ]
+
+        if x_axis:
+            sections.insert(0, "[x-axis]")
+
+        ini_content = "\n".join(sections)
+        with open(f"{ininame}.ini", "w") as file:
+            file.write(ini_content)
+        print(f"Successfully generated {ininame}.ini\n")
+        return f"{ininame}.ini"
+    except Exception as err:
+        print(f"Error producing ini file: {err}\n")
+        return None
+
+
+def read_annotation_bed(filepath):
+    """Reads a BED file into a Pandas DataFrame and ensures correct formatting."""
+    col_names = [
+        "chrom",
+        "start",
+        "end",
+        "name",
+        "score",
+        "strand",
+        "thickStart",
+        "thickEnd",
+        "itemRgb",
+    ]  # Include additional fields
+
+    df = pd.read_csv(filepath, sep="\t", comment="#", header=None)
+
+    # Ensure at least three required columns exist
+    if df.shape[1] < 3:
+        raise ValueError(
+            "Invalid BED file: must have at least 3 columns (chrom, start, end)."
+        )
+
+    # Rename only the expected columns
+    df.columns = col_names[: df.shape[1]]
+
+    # Ensure start and end columns contain valid integers
+    if df["start"].isna().any() or df["end"].isna().any():
+        raise ValueError(
+            "Invalid BED file: 'start' and 'end' columns must be integers and contain no missing values."
+        )
+
+    return df
+
+
+def make_svg_background_transparent(svg_path, output_path=None):
+    """
+    Makes the background of an SVG file transparent by removing/modifying background fills.
+
+    Args:
+        svg_path: Path to input SVG file
+        output_path: Path to output SVG file (if None, overwrites input)
+    """
+    import xml.etree.ElementTree as ET
+    import re
+
+    if output_path is None:
+        output_path = svg_path
+
+    # Parse the SVG
+    tree = ET.parse(svg_path)
+    root = tree.getroot()
+
+    # Define SVG namespace
+    ns = {"svg": "http://www.w3.org/2000/svg"}
+
+    # Remove background rectangles/paths that cover the entire canvas
+    # Get SVG dimensions for comparison
+    width = root.get("width", "0")
+    height = root.get("height", "0")
+
+    # Extract numeric values
+    width_num = float(re.sub(r"[a-zA-Z%]+", "", width)) if width != "0" else 0
+    height_num = float(re.sub(r"[a-zA-Z%]+", "", height)) if height != "0" else 0
+
+    # Find and modify background elements
+    elements_to_modify = []
+
+    # Check all paths, rectangles, and other elements
+    for elem in root.iter():
+        if (
+            elem.tag.endswith("path")
+            or elem.tag.endswith("rect")
+            or elem.tag.endswith("polygon")
+        ):
+            # Check if this element has a background-like fill
+            style = elem.get("style", "")
+            fill = elem.get("fill", "")
+
+            # Look for background colors (light colors, white, etc.)
+            background_colors = [
+                "#ffffff",
+                "#f0ffff",
+                "white",
+                "lightblue",
+                "lightgray",
+                "lightgrey",
+            ]
+
+            is_background = False
+            current_fill = None
+
+            if "fill:" in style:
+                # Extract fill from style
+                fill_match = re.search(r"fill:\s*([^;]+)", style)
+                if fill_match:
+                    current_fill = fill_match.group(1).strip()
+            elif fill:
+                current_fill = fill
+
+            if current_fill and any(
+                bg_color in current_fill.lower() for bg_color in background_colors
+            ):
+                is_background = True
+
+            # For paths, check if it covers a large area (likely background)
+            if elem.tag.endswith("path"):
+                d = elem.get("d", "")
+                # Simple heuristic: if path starts at 0,0 and covers large area, it's likely background
+                if "M 0" in d and current_fill:
+                    is_background = True
+
+            # For rectangles, check if it covers the full canvas
+            if elem.tag.endswith("rect"):
+                x = float(elem.get("x", 0))
+                y = float(elem.get("y", 0))
+                w = float(elem.get("width", 0))
+                h = float(elem.get("height", 0))
+
+                # If rectangle covers most/all of the canvas, it's likely background
+                if x <= 1 and y <= 1 and w >= width_num * 0.9 and h >= height_num * 0.9:
+                    is_background = True
+
+            if is_background:
+                elements_to_modify.append(elem)
+
+    # Modify the background elements
+    for elem in elements_to_modify:
+        style = elem.get("style", "")
+
+        if "fill:" in style:
+            # Replace fill in style
+            new_style = re.sub(r"fill:\s*[^;]+", "fill: transparent", style)
+            elem.set("style", new_style)
+        elif elem.get("fill"):
+            # Replace fill attribute
+            elem.set("fill", "transparent")
+
+    # Save the modified SVG
+    tree.write(output_path, encoding="unicode", xml_declaration=True)
+
+
+def make_all_svg_backgrounds_transparent(directory):
+    """
+    Makes all SVG files in a directory have transparent backgrounds.
+    """
+
+    svg_files = glob.glob(os.path.join(directory, "*.svg"))
+
+    for svg_file in svg_files:
+        make_svg_background_transparent(svg_file)
+
+    print(f"Processed {len(svg_files)} SVG files")
+
+
+def run_pygenometracks(inifile, region, output_file, width):
+    output_dir = os.path.dirname(output_file)
+    if output_dir and not os.path.exists(output_dir):
+        os.makedirs(output_dir)  # Create directory if it doesn't exist
+
+    try:
+        trp = PlotTracks(
+            inifile,
+            width,
+            fig_height=None,
+            fontsize=10,
+            dpi=300,
+            track_label_width=0.05,
+            plot_regions=region,
+            plot_width=width,
+        )
+
+        # Extract the chromosome, start, and end from the region
+        chrom, start, end = region[0]
+
+        # Call the plot method directly to generate the image
+        fig = trp.plot(output_file, chrom, start, end)
+
+        return trp
+
+    except Exception as e:
+        if "No valid intervals were found" in str(e):
+            print(f"No valid intervals found in BED file for region {region[0]}")
+            print(
+                "This is expected when the BED file doesn't overlap with the query region."
+            )
+            return None
+        else:
+            print(f"Error in run_pygenometracks: {e}")
+            raise e
+
+
+def test_pygenometracks_direct(inifile, chrom, start, end, output_file, width=40):
+    """
+    Direct test function to call PlotTracks.plot() without any coordinate validation.
+    This bypasses the region checking that happens during PlotTracks initialization.
+
+    Args:
+        inifile: Path to the tracks configuration file
+        chrom: Chromosome name
+        start: Start coordinate
+        end: End coordinate
+        output_file: Output image file path
+        width: Figure width in cm
+    """
+
+    output_dir = os.path.dirname(output_file)
+    if output_dir and not os.path.exists(output_dir):
+        os.makedirs(output_dir)
+
+    # Create a dummy region for initialization (this gets overridden in plot())
+    dummy_region = [(chrom, 1, 1000)]
+
+    trp = PlotTracks(
+        inifile,
+        width,
+        fig_height=None,
+        fontsize=10,
+        dpi=300,
+        track_label_width=0.05,
+        plot_regions=dummy_region,
+        plot_width=width,
+    )
+
+    # Call plot() directly with your desired coordinates
+    fig = trp.plot(output_file, chrom, start, end)
+
+    return fig
+
+
 def reverse_pascal(double_vals):
     if len(double_vals) == 1:
         return 2
@@ -137,43 +415,6 @@ def make_scale(vals: list) -> list:
         return make_m(scaled)
 
 
-def overlap_axis(rotated_plot, filename, prefix):
-    scale_factor = math.sqrt(2) + 0.04
-    new_width = int(rotated_plot.width / scale_factor)
-    new_height = int(rotated_plot.height / scale_factor)
-    resized_rotated_plot = rotated_plot.resize((new_width, new_height), Image.LANCZOS)
-
-    # Step 3: Overlay the resized rotated heatmap onto the original axes
-
-    # Open the original heatmap with axes
-    image_with_axes = Image.open(filename)
-
-    # Create a blank image with the same size as the original
-    final_image = Image.new("RGBA", image_with_axes.size)
-
-    # Calculate the position to center the resized rotated image within the original plot area
-    x_offset = (final_image.width - resized_rotated_plot.width) // 2
-    y_offset = (final_image.height - resized_rotated_plot.height) // 2
-    y_offset += 2400
-    x_offset += 30
-
-    # Paste the original image with axes onto the final image
-    final_image.paste(image_with_axes, (0, 0))
-
-    # Paste the resized rotated plot onto the final image
-    final_image.paste(resized_rotated_plot, (x_offset, y_offset), resized_rotated_plot)
-    width, height = final_image.size
-    cropped_image = final_image.crop((0, height // 2.6, width, height))
-
-    # Save or show the final image
-    cropped_image.save(f"{prefix}_TRI.png")
-    cropped_image.save(f"{prefix}_TRI.pdf", "PDF", resolution=100.0)
-
-    # Remove temp files
-    if os.path.exists(filename):
-        os.remove(filename)
-
-
 def get_colors(sdf, ncolors, is_freq, custom_breakpoints):
     assert ncolors > 2 and ncolors < 12
     try:
@@ -190,7 +431,7 @@ def get_colors(sdf, ncolors, is_freq, custom_breakpoints):
     else:
         breaks = [bot + i * interval for i in range(ncolors + 1)]
     if custom_breakpoints:
-        breaks = np.asfarray(custom_breakpoints)
+        np.asarray(custom_breakpoints, dtype=np.float64)
     labels = np.arange(len(breaks) - 1)
     # corner case of only one %id value
     if len(breaks) == 1:
@@ -282,32 +523,35 @@ def read_df(
     return df
 
 
-def generate_breaks(number, min_breaks=6, max_breaks=8):
+def generate_breaks(min_number, max_number, min_breaks=5, max_breaks=9):
     # Determine the order of magnitude
-    magnitude = 10 ** int(
-        np.floor(np.log10(number))
-    )  # Base power of 10 (e.g., 10M, 1M, 100K)
+    difference = max_number - min_number
 
-    # Find a reasonable step size
-    possible_steps = [
-        magnitude // d
-        for d in [10, 5, 4, 2, 1]
-        if (number // (magnitude // d)) in range(min_breaks, max_breaks + 1)
-    ]
+    magnitude = 10 ** int(math.floor(math.log10(difference)))
+    threshold = math.ceil(difference / magnitude)
 
-    step = (
-        possible_steps[0] if possible_steps else magnitude // 5
-    )  # Default to 1/5th if no exact match
+    while threshold > max_breaks:
+        magnitude *= 2
+        threshold = math.ceil(difference / magnitude)
+
+    while threshold < min_breaks:
+        magnitude /= 2
+        threshold = math.ceil(difference / magnitude)
+
+    # Round down min_number to the nearest multiple of magnitude
+    min_aligned = int(min_number // magnitude * magnitude)
 
     # Generate breakpoints
-    breaks = list(range(0, ((number // step) + 1) * step, step))
+    upper_bound = int(min_aligned + (threshold + 1) * magnitude)
+    breaks = list(range(min_aligned, upper_bound, int(magnitude)))
 
     return breaks
 
 
 def make_dot(
     sdf,
-    title_name,
+    name_x,
+    name_y,
     palette,
     palette_orientation,
     colors,
@@ -316,7 +560,17 @@ def make_dot(
     xlim,
     deraster,
     width,
+    is_pairwise,
 ):
+    if is_pairwise:
+        title_name = f"Comparative Plot: {name_x} vs {name_y}"
+    else:
+        title_name = f"Self-Identity Plot: {name_x}"
+    title_length = 2 * width
+    if len(title_name) > 50:
+        title_length = 1.5 * width
+    elif len(title_name) > 80:
+        title_length = width
     # Select the color palette
     if hasattr(diverging, palette):
         function_name = getattr(diverging, palette)
@@ -340,11 +594,12 @@ def make_dot(
     if not xlim:
         xlim = 0
     # Determine maximum genomic position for scaling
+    min_val = min(sdf["q_st"].min(), sdf["r_st"].min())
     max_val = max(sdf["q_en"].max(), sdf["r_en"].max(), xlim)
 
     # If user provides breaks, convert to ints
     if not breaks:
-        breaks = generate_breaks(int(max_val))
+        breaks = generate_breaks(int(min_val), int(max_val))
     else:
         [int(x) for x in breaks]
     xlim = xlim or 0
@@ -374,8 +629,10 @@ def make_dot(
         axis_ticks_major=element_line(
             size=(width), color="black"
         ),  # Increased tick length
-        title=element_blank(),  # Center title
-        axis_title_x=element_text(size=(width * 1.2), family=["DejaVu Sans"]),
+        title=element_text(
+            family=["DejaVu Sans"], size=title_length, hjust=0.5
+        ),  # Center title
+        axis_title_x=element_text(size=(width * 1.4), family=["DejaVu Sans"]),
         strip_background=element_blank(),  # Remove facet strip background
         strip_text=element_text(
             size=(width * 1.2), family=["DejaVu Sans"]
@@ -388,8 +645,12 @@ def make_dot(
         + scale_color_discrete(guide=False)
         + scale_fill_manual(values=new_hexcodes, guide=False)
         + common_theme
-        + scale_x_continuous(labels=make_scale, limits=[0, max_val], breaks=breaks)
-        + scale_y_continuous(labels=make_scale, limits=[0, max_val], breaks=breaks)
+        + scale_x_continuous(
+            labels=make_scale, limits=[min_val, max_val], breaks=breaks
+        )
+        + scale_y_continuous(
+            labels=make_scale, limits=[min_val, max_val], breaks=breaks
+        )
         + coord_fixed(ratio=1)
         + facet_grid("r ~ q")
         + labs(x=x_label, y="", title=title_name)
@@ -438,11 +699,12 @@ def make_dot_grid(
     if not xlim:
         xlim = 0
     # Determine maximum genomic position for scaling
+    min_val = max(sdf["q_st"].min(), sdf["r_st"].min())
     max_val = max(sdf["q_en"].max(), sdf["r_en"].max(), xlim)
 
     # If user provides breaks, convert to ints
     if not breaks:
-        breaks = generate_breaks(int(max_val))
+        breaks = generate_breaks(int(min_val), int(max_val))
     else:
         [int(x) for x in breaks]
     xlim = xlim or 0
@@ -533,11 +795,12 @@ def make_dot_final(
     if not xlim:
         xlim = 0
     # Determine maximum genomic position for scaling
+    min_val = min(sdf["q_st"].min(), sdf["r_st"].min())
     max_val = max(sdf["q_en"].max(), sdf["r_en"].max(), xlim)
 
     # If user provides breaks, convert to ints
     if not breaks:
-        breaks = generate_breaks(int(max_val))
+        breaks = generate_breaks(int(min_val), int(max_val))
     else:
         [int(x) for x in breaks]
     xlim = xlim or 0
@@ -647,11 +910,12 @@ def make_tri(
     if not xlim:
         xlim = 0
     # Determine maximum genomic position for scaling
+    min_val = max(sdf["q_st"].min(), sdf["r_st"].min())
     max_val = max(sdf["q_en"].max(), sdf["r_en"].max(), xlim)
 
     # If user provides breaks, convert to ints
     if not breaks:
-        breaks = generate_breaks(int(max_val))
+        breaks = generate_breaks(int(min_val), int(max_val))
     else:
         [int(x) for x in breaks]
     xlim = xlim or 0
@@ -678,8 +942,12 @@ def make_tri(
             )  # Ensure full opacity
             + scale_fill_manual(values=new_hexcodes, guide=False)
             + scale_color_discrete(guide=False)
-            + scale_x_continuous(labels=make_scale, limits=[0, max_val], breaks=breaks)
-            + scale_y_continuous(labels=make_scale, limits=[0, max_val], breaks=breaks)
+            + scale_x_continuous(
+                labels=make_scale, limits=[min_val, max_val], breaks=breaks
+            )
+            + scale_y_continuous(
+                labels=make_scale, limits=[min_val, max_val], breaks=breaks
+            )
             + coord_fixed(ratio=1)
             + labs(x=x_label, y="", title=title_name)
             + theme(
@@ -695,7 +963,7 @@ def make_tri(
                 axis_ticks_major_y=element_blank(),
                 axis_ticks_major=element_line(size=(width)),
                 title=element_text(size=(width * 1.4), hjust=0.5),
-                axis_title_x=element_text(size=(width * 1.2), family=["DejaVu Sans"]),
+                axis_title_x=element_text(size=(width * 1.4), family=["DejaVu Sans"]),
                 axis_text_y=element_blank(),
             )
         )
@@ -739,8 +1007,12 @@ def make_tri(
             )  # Ensure full opacity
             + scale_fill_manual(values=new_hexcodes, guide=False)
             + scale_color_discrete(guide=False)
-            + scale_x_continuous(labels=make_scale, limits=[0, max_val], breaks=breaks)
-            + scale_y_continuous(labels=make_scale, limits=[0, max_val], breaks=breaks)
+            + scale_x_continuous(
+                labels=make_scale, limits=[min_val, max_val], breaks=breaks
+            )
+            + scale_y_continuous(
+                labels=make_scale, limits=[min_val, max_val], breaks=breaks
+            )
             + coord_fixed(ratio=1)
             + labs(x=x_label, y="", title=title_name)
             + theme(
@@ -768,8 +1040,12 @@ def make_tri(
             )
             + scale_color_discrete(guide=False)
             + scale_fill_manual(values=new_hexcodes, guide=False)
-            + scale_x_continuous(labels=make_scale, limits=[0, max_val], breaks=breaks)
-            + scale_y_continuous(labels=make_scale, limits=[0, max_val], breaks=breaks)
+            + scale_x_continuous(
+                labels=make_scale, limits=[min_val, max_val], breaks=breaks
+            )
+            + scale_y_continuous(
+                labels=make_scale, limits=[min_val, max_val], breaks=breaks
+            )
             + coord_fixed(ratio=1)
             + labs(x="", y="", title="")
             + theme(
@@ -888,6 +1164,100 @@ def rotate_rasterized_tri(svg_path, shift_x, shift_y):
     tree.write(svg_path)
 
 
+from lxml import etree
+
+
+def append_svg(svg1_path, svg2_path, output_path):
+    """Appends SVG2 to the bottom of SVG1 without modifying its width."""
+    # Load SVG1 and SVG2
+    tree1 = etree.parse(svg1_path)
+    root1 = tree1.getroot()
+
+    tree2 = etree.parse(svg2_path)
+    root2 = tree2.getroot()
+
+    # Extract width and height of SVG1
+    width1 = float(root1.get("width", "0").replace("pt", ""))
+    height1 = float(root1.get("height", "0").replace("pt", ""))
+
+    # Extract width and height of SVG2
+    width2 = float(root2.get("width", "0").replace("pt", ""))
+    height2 = float(root2.get("height", "0").replace("pt", ""))
+
+    # Update SVG1 height to accommodate SVG2
+    new_height = height1 + height2
+    root1.set("height", f"{new_height}pt")
+
+    # Create a translation group for SVG2 and shift it down
+    group = etree.Element("g", attrib={"transform": f"translate(0,{height1 + 400})"})
+    for child in root2:
+        group.append(child)
+
+    # Append translated SVG2 to SVG1
+    root1.append(group)
+
+    # Save the new merged SVG
+    tree1.write(output_path, pretty_print=True, xml_declaration=True, encoding="utf-8")
+
+
+def get_svg_size(svg_path):
+    """Helper to extract width and height of an SVG in pt units."""
+    import xml.etree.ElementTree as ET
+
+    tree = ET.parse(svg_path)
+    root = tree.getroot()
+    width = root.get("width")
+    height = root.get("height")
+    return width, height
+
+
+def parse_size(size_str):
+    """Convert '648pt' or '800px' -> float(648)."""
+    return float(re.sub(r"[a-zA-Z]+", "", size_str))
+
+
+def merge_annotation_tri(svg1_path, svg2_path, output_path, deraster, width):
+    """Merges two SVG files into a single SVG file with proper size."""
+
+    w1, h1 = get_svg_size(svg1_path)
+    w2, h2 = get_svg_size(svg2_path)
+
+    # Ensure they are floats
+    w1, h1 = parse_size(w1), parse_size(h1)
+    w2, h2 = parse_size(w2), parse_size(h2)
+    # Determine total size
+    total_width = max(w1, w2)
+    total_height = h1 + h2
+    # Create figure
+    fig = sg.SVGFigure(f"{total_width}px", f"{total_height}px")
+
+    # Load SVGs
+    svg1 = sg.fromfile(svg1_path).getroot()
+    svg2 = sg.fromfile(svg2_path).getroot()
+    make_svg_background_transparent(svg2_path)
+
+    # Position
+    if deraster:
+        # Its not perfect for width > 18, but good enough
+        adjust_svg1 = (0, -5 * (h1 / 6))
+        adjust_svg2 = ((9 - (width / 2)), 5 * (h1 / 6))
+        svg1.moveto(adjust_svg1[0], adjust_svg1[1])
+        svg2.scale(1.077 + (width / 1000))
+        svg2.moveto(adjust_svg2[0], adjust_svg2[1])
+    else:
+        adjust_svg1 = (0, -5 * (h1 / 6))
+        adjust_svg2 = (10 + (width / 4.5), 5 * (h1 / 6))
+        svg1.moveto(adjust_svg1[0], adjust_svg1[1])
+        svg2.moveto(adjust_svg2[0], adjust_svg2[1])
+        scaling_factor = width / 2
+        svg2.scale(1.034 + (scaling_factor / 1000))
+
+    # Append and save
+    fig.append([svg1, svg2])
+    fig.set_size((f"{total_width}px", f"{total_height}px"))
+    fig.save(output_path)
+
+
 def make_tri_axis(sdf, title_name, palette, palette_orientation, colors, breaks, xlim):
     if not breaks:
         breaks = True
@@ -1340,6 +1710,7 @@ def create_plots(
     axes_tick_number,
     vector_format,
     deraster,
+    annotation,
 ):
     df = read_df(
         sdf,
@@ -1351,18 +1722,78 @@ def create_plots(
         from_file,
     )
     sdf = df
-    plot_filename = f"{directory}/{name_x}"
+
+    plot_filename = os.path.join(directory, name_x)
+
     if is_pairwise:
-        plot_filename = f"{directory}/{name_x}_{name_y}"
+        plot_filename = os.path.join(directory, f"{name_x}_{name_y}")
 
     histy = make_hist(
         sdf, palette, palette_orientation, custom_colors, custom_breakpoints
     )
 
+    # Just doing triangle plots for now.
+    if annotation:
+        print(f"Generating ini file for annotation track:\n")
+        iniprefix = plot_filename
+        inifile = generate_ini_file(
+            bedfile=annotation,
+            ininame=iniprefix,
+            chrom=name_x,
+        )
+        min_val = max(sdf["q_st"].min(), sdf["r_st"].min())
+        max_val = max(sdf["q_en"].max(), sdf["r_en"].max())
+        if not xlim:
+            xlim = max_val
+        region = [(name_x.split(":")[0], min_val, xlim)]
+        if inifile:
+            try:
+                # Check if the BED file has valid intervals for this region first
+                bed_df = read_annotation_bed(annotation)
+                chrom_name = name_x.split(":")[0]
+
+                # Filter for the chromosome and region of interest
+                valid_intervals = bed_df[
+                    (bed_df["chrom"] == chrom_name)
+                    & (bed_df["end"] >= min_val)
+                    & (bed_df["start"] <= xlim)
+                ]
+
+                if valid_intervals.empty:
+                    print(
+                        f"No valid intervals found in {annotation} for region {chrom_name}:{min_val}-{xlim}.\n"
+                    )
+                    print("Skipping annotation track generation.\n")
+                else:
+                    bed_track = run_pygenometracks(
+                        inifile=inifile,
+                        region=region,
+                        output_file=f"{iniprefix}_ANNOTATION_TRACK.svg",
+                        width=width * 2.05,
+                    )
+                    bed_track.plot(
+                        f"{iniprefix}_ANNOTATION_TRACK.svg",
+                        name_x.split(":")[0],
+                        min_val,
+                        xlim,
+                    )
+                    bed_track.plot(
+                        f"{iniprefix}_ANNOTATION_TRACK.png",
+                        name_x.split(":")[0],
+                        min_val,
+                        xlim,
+                    )
+                    print(f"\nAnnotation track saved to {iniprefix}_ANNOTATION_TRACK\n")
+
+            except Exception as e:
+                print(f"Error processing annotation file {annotation}: {e}\n")
+                print("Skipping annotation track generation.\n")
+
     if is_pairwise:
         heatmap = make_dot(
             sdf,
-            plot_filename,
+            name_x,
+            name_y,
             palette,
             palette_orientation,
             custom_colors,
@@ -1371,6 +1802,7 @@ def create_plots(
             xlim,
             deraster,
             width,
+            True,
         )
         print(f"Creating plots and saving to {plot_filename}...\n")
         ggsave(
@@ -1391,29 +1823,14 @@ def create_plots(
             filename=f"{plot_filename}_COMPARE.png",
             verbose=False,
         )
-        try:
-            if not heatmap.data:
-                print(
-                    f"{plot_filename}_COMPARE.pdf and {plot_filename}_COMPARE.png saved sucessfully. \n"
-                )
-                return 0
-        except ValueError:
-            print(
-                f"{plot_filename}_COMPARE.pdf and {plot_filename}_COMPARE.png saved sucessfully. \n"
-            )
-            return 0
-        if no_hist:
-            print(
-                f"{plot_filename}_COMPARE.pdf and {plot_filename}_COMPARE.png saved sucessfully. \n"
-            )
-        else:
+        if not no_hist:
             ggsave(
                 histy,
                 width=3,
                 height=3,
                 dpi=dpi,
                 format=vector_format,
-                filename=f"{plot_filename}_HIST.{vector_format}",
+                filename=f"{plot_filename}_COMPARE_HIST.{vector_format}",
                 verbose=False,
             )
             ggsave(
@@ -1422,16 +1839,30 @@ def create_plots(
                 height=3,
                 dpi=dpi,
                 format="png",
-                filename=f"{plot_filename}_HIST.png",
+                filename=f"{plot_filename}_COMPARE_HIST.png",
                 verbose=False,
             )
+        try:
+            if not heatmap.data:
+                print(
+                    f"{plot_filename} comparative plots and histogram saved sucessfully. \n"
+                )
+                return 0
+        except ValueError:
             print(
-                f"{plot_filename}_COMPARE.pdf, {plot_filename}_COMPARE.png, {plot_filename}_HIST.pdf and {plot_filename}_HIST.png saved sucessfully. \n"
+                f"{plot_filename} comparative plots and histogram saved sucessfully. \n"
+            )
+            return 0
+        if no_hist:
+            print(
+                f"{plot_filename}_COMPARE.{vector_format} and {plot_filename}_COMPARE.png saved sucessfully. \n"
             )
     # Self-identity plots: Output _TRI, _FULL, and _HIST
     else:
         if deraster:
-            print(f"Derasterization turned off. This may take a while...\n")
+            print(
+                f"Producing dotplots with derasterization turned off. This may take a while...\n"
+            )
         tri_plot = make_tri(
             sdf,
             plot_filename,
@@ -1446,7 +1877,8 @@ def create_plots(
         )
         full_plot = make_dot(
             check_st_en_equality(sdf),
-            plot_filename,
+            name_x,
+            name_y,
             palette,
             palette_orientation,
             custom_colors,
@@ -1455,6 +1887,7 @@ def create_plots(
             xlim,
             deraster,
             width,
+            False,
         )
         ggsave(
             full_plot,
@@ -1484,13 +1917,34 @@ def create_plots(
             filename=f"{tri_prefix}.svg",
             verbose=False,
         )
-
+        if annotation:
+            anno_prefix = f"{plot_filename}_PRE_ANNOTATED"
+            annotated_tri = tri_plot[0] + theme(
+                axis_title_x=element_blank(),
+                axis_line_x=element_blank(),
+                axis_text_x=element_blank(),
+                axis_ticks_minor_x=element_blank(),
+                axis_ticks=element_blank(),
+            )
+            ggsave(
+                annotated_tri,
+                width=width,
+                height=width,
+                dpi=dpi,
+                format="svg",
+                filename=f"{anno_prefix}.svg",
+                verbose=False,
+            )
         # These scaling values were determined thorugh much trial and error. Please don't delete :)
         if deraster:
             scaling_values = (46.62 * width, -3.75 * width)
             rotate_vectorized_tri(
                 f"{tri_prefix}.svg", scaling_values[0], scaling_values[1]
             )
+            if annotation:
+                rotate_vectorized_tri(
+                    f"{anno_prefix}.svg", scaling_values[0], scaling_values[1]
+                )
             try:
                 cairosvg.svg2png(
                     url=f"{tri_prefix}.svg", write_to=f"{tri_prefix}.png", dpi=dpi
@@ -1502,13 +1956,58 @@ def create_plots(
             rotate_rasterized_tri(
                 f"{tri_prefix}.svg", scaling_values[0], scaling_values[1]
             )
+            if annotation:
+                if annotation:
+                    rotate_rasterized_tri(
+                        f"{anno_prefix}.svg", scaling_values[0], scaling_values[1]
+                    )
             try:
                 cairosvg.svg2png(
                     url=f"{tri_prefix}.svg", write_to=f"{tri_prefix}.png", dpi=dpi
                 )
             except:
                 print(f"Error installing cairosvg. Unable to convert svg file. \n")
-
+        if annotation:
+            # Only merge if annotation was successfully created
+            if os.path.exists(f"{iniprefix}_ANNOTATION_TRACK.svg"):
+                make_svg_background_transparent(f"{iniprefix}_ANNOTATION_TRACK.svg")
+                merge_annotation_tri(
+                    f"{anno_prefix}.svg",
+                    f"{iniprefix}_ANNOTATION_TRACK.svg",
+                    f"{tri_prefix}_ANNOTATED.svg",
+                    deraster,
+                    width,
+                )
+                if os.path.exists(f"{anno_prefix}.svg"):
+                    os.remove(f"{anno_prefix}.svg")
+                try:
+                    if vector_format != "svg":
+                        if vector_format == "pdf":
+                            cairosvg.svg2pdf(
+                                url=f"{tri_prefix}_ANNOTATED.svg",
+                                write_to=f"{tri_prefix}_ANNOTATED.pdf",
+                            )
+                            cairosvg.svg2pdf(
+                                url=f"{iniprefix}_ANNOTATION_TRACK.svg",
+                                write_to=f"{iniprefix}_ANNOTATION_TRACK.pdf",
+                            )
+                        elif vector_format == "ps":
+                            cairosvg.svg2ps(
+                                url=f"{tri_prefix}_ANNOTATED.svg",
+                                write_to=f"{tri_prefix}_ANNOTATED.ps",
+                            )
+                            cairosvg.svg2ps(
+                                url=f"{tri_prefix}_ANNOTATION_TRACK.svg",
+                                write_to=f"{tri_prefix}_ANNOTATION_TRACK.ps",
+                            )
+                        if os.path.exists(f"{iniprefix}_ANNOTATION_TRACK.svg"):
+                            os.remove(f"{iniprefix}_ANNOTATION_TRACK.svg")
+                        if os.path.exists(f"{tri_prefix}_ANNOTATED.svg"):
+                            os.remove(f"{tri_prefix}_ANNOTATED.svg")
+                except Exception as e:
+                    print(f"Error converting annotated SVG: {e}")
+            else:
+                print("Annotation file not created, skipping merge step.")
         # Convert from svg to selected vector format. Ignore error if user has issues with cairosvg.
         try:
             if vector_format != "svg":
